Package: varGuidTS
Version: 0.1.13
Date: 2026-05-20
Title: Variance-Guided Time-Series Modeling for Temporal Risk Detection
Authors@R: c(person("Zihao", "Wang", role = "aut"),
  person("Min", "Lu", email = "luminwin@gmail.com", role = c("aut", "cre")))
Author: Zihao Wang [aut],
  Min Lu [aut, cre]
Maintainer: Min Lu <luminwin@gmail.com>
BugReports: https://github.com/zionwzz/variance-guided-risk-demo/issues
Depends: R (>= 4.1.0)
Imports: stats, glmnet
Suggests: testthat (>= 3.0.0)
Description: Fits balanced-panel autoregressive models with conditional
  heteroscedasticity for temporal risk detection. The main estimator combines
  autoregressive exogenous mean modeling with GARCH-X variance modeling,
  subject-specific baseline terms, shared population coefficients, and L1
  penalization for high-dimensional covariates. The package returns
  conditional mean and variance estimates, coefficient summaries, simulations,
  and exceedance-based risk scores defined as estimated conditional
  threshold-exceedance probabilities. The implementation builds on the lasso
  of Tibshirani (1996) <doi:10.1111/j.2517-6161.1996.tb02080.x>,
  generalized autoregressive conditional heteroscedasticity of Bollerslev
  (1986) <doi:10.1016/0304-4076(86)90063-1>, and L1-regularized
  high-dimensional time-series modeling of Medeiros and Mendes (2016)
  <doi:10.1016/j.jeconom.2015.10.011>.
License: MIT + file LICENSE
URL: https://github.com/zionwzz/variance-guided-risk-demo
Encoding: UTF-8
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-20 18:10:00 UTC; minlu
Repository: CRAN
Date/Publication: 2026-05-28 11:10:02 UTC
Built: R 4.4.3; ; 2026-05-28 11:51:22 UTC; unix

# Generated by hand to match the documented API.

export(lmvt)
export(simulate_scenario)

S3method(predict, lmvt)
S3method(print,   lmvt)
S3method(coef,    lmvt)
S3method(summary, lmvt)
S3method(print,   summary.lmvt)

importFrom(stats,
           coef, lm, lm.wfit, median, optim, pnorm, pt, quantile,
           rnorm, rt, sd, setNames, var)
importFrom(glmnet, glmnet, cv.glmnet)

# varGuidTS

> Variance-Guided Time-Series Modeling for Temporal Risk Detection

`varGuidTS` fits a balanced panel ARX-GARCHX model with subject-specific
intercepts,
shared population coefficients, and joint $\ell_1$ penalization on the
conditional mean and variance covariate vectors. The fitted object also
exposes an exceedance-based risk score

$$
\hat\pi_{s,t}(c) = P\!\left(Y_{s,t} > c \;\middle|\; \mathcal{F}_{t-1}\right),
$$

useful as a personalized, scale-normalized temporal risk score from
high-frequency multimodal panel data such as wearable-sensor streams.

## Installation

```r
# from GitHub
install.packages("remotes")
remotes::install_github("zionwzz/variance-guided-risk-demo")

# or from a local source tree
remotes::install_local("path/to/varGuidTS")
```

Project URL: <https://github.com/zionwzz/variance-guided-risk-demo>

## Quick start

```r
library(varGuidTS)

set.seed(1)
sim <- simulate_scenario(scen = 2, S = 4, T = 60,
                         d_noise = 4, noise_kind = "ar1", seed = 1)
df  <- sim[, c("s", "t", "y",
               grep("^X|^Noise|^Xbin", names(sim), value = TRUE))]

fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
            lambda_beta = 0.05, lambda_gamma = 0.05,
            use_x_in_variance = TRUE, maxit = 8)
print(fit)

pred <- predict(fit, df, threshold = quantile(df$y, 0.9))
head(pred)

# Coefficient summary table
sm <- summary(fit, include_zero = FALSE, sort_by = "abs_estimate")
head(sm$table)

# Subject-specific thresholds for personalized exceedance probabilities
subject_cutoffs <- tapply(df$y, df$s, quantile, probs = 0.75, na.rm = TRUE)
pred_subject <- predict(fit, df, threshold = subject_cutoffs,
                        innov_g = TRUE, innov_t = TRUE, df_t = 8)
head(pred_subject)
```

The README quick start gives the main workflow; the simulation function can be used to generate built-in scenarios for examples, tests, and method demonstrations.

## What the package provides

| Function | Purpose |
|---|---|
| `lmvt()` | Fit penalized panel ARX–GARCHX |
| `predict.lmvt()` | Conditional mean, variance, and exceedance probability |
| `summary.lmvt()` | Model diagnostics and coefficient summary table |
| `simulate_scenario()` | Generate panels under the four canonical scenarios |

## Maintainer

Min Lu <luminwin@gmail.com>.

## License

MIT.

## Citation

Please cite the package repository or the CRAN package record when using
`varGuidTS`.

## ============================================================================
## Package-level documentation stub. Hand-written Rd lives in man/.
## ============================================================================

#' varGuidTS: Variance-Guided Time-Series Modeling for Temporal Risk Detection
#'
#' Fits a balanced panel ARX-GARCHX model with subject-specific
#' intercepts, shared population coefficients, and joint L1
#' penalization on the conditional mean and variance covariate vectors.
#' Provides an exceedance-based risk score
#' \eqn{\pi_{s,t}(c) = P(Y_{s,t} > c \,|\, F_{t-1})} for personalized
#' temporal risk assessment.
#'
#' @keywords internal
"_PACKAGE"

# Internal utilities for varGuidTS.
# Not exported. Lightweight helpers used by lmvt() and predict.lmvt().

`%||%` <- function(x, y) if (is.null(x)) y else x

# Lag a numeric vector v within each subject, where ids and a sorted
# (s, t) data frame `df` are passed in. Returns NA for the first L
# observations of each subject.
.lag_by_id <- function(v, L, s_vec, ids) {
    out <- rep(NA_real_, length(v))
    if (L <= 0L) return(v)
    for (id in ids) {
        idx <- which(s_vec == id)
        if (length(idx) > L) {
            out[idx[(L + 1L):length(idx)]] <- v[idx[1:(length(idx) - L)]]
        }
    }
    out
}

# Soft-thresholding operator used in proximal updates.
.soft_threshold <- function(z, lambda) sign(z) * pmax(abs(z) - lambda, 0)

# Build the lagged-X design block: cbind(X, lag(X, 1), ..., lag(X, q)).
.lagged_X <- function(X, q, s_vec, ids) {
    if (q == 0L) return(X)
    blocks <- vector("list", q + 1L)
    blocks[[1L]] <- X
    for (L in seq_len(q)) {
        blocks[[L + 1L]] <- apply(X, 2L, .lag_by_id, L = L,
                                  s_vec = s_vec, ids = ids)
    }
    do.call(cbind, blocks)
}

## ============================================================================
## simulate_scenario() : data generators for the four canonical scenarios
## used by the package examples and tests.
##
## Ported from `simulator_scenarios.r` in the demo repository, with
## ggplot2/dplyr dependencies removed (plotting helpers live in
## inst/scripts/ instead).
## ============================================================================

# Standardized Student-t (unit variance)
.rstdt <- function(n, nu) {
    stopifnot(nu > 2)
    stats::rt(n, df = nu) / sqrt(nu / (nu - 2))
}
.pstdt <- function(q, nu) {
    stopifnot(nu > 2)
    stats::pt(q * sqrt(nu / (nu - 2)), df = nu)
}

# Piecewise regime path around a target run length L.
.gen_regime_piecewise <- function(Ttot, L,
                                  min_run = max(1L, floor(0.5 * L)),
                                  jitter  = floor(0.3 * L)) {
    reg <- integer(Ttot); t <- 1L; state <- 0L
    while (t <= Ttot) {
        run <- max(min_run,
                   L + sample.int(2L * jitter + 1L, 1L) - (jitter + 1L))
        t_to <- min(Ttot, t + run - 1L)
        reg[t:t_to] <- state
        state <- 1L - state
        t <- t_to + 1L
    }
    reg
}

# Covariate generators
.gen_iid <- function(S, Ttot, d) {
    array(stats::rnorm(S * Ttot * d), c(S, Ttot, d))
}
.gen_ar1_cov <- function(S, Ttot, d, phi = 0.6, sigma_eta = 1) {
    phi <- rep(phi, length.out = d)
    sigma_eta <- rep(sigma_eta, length.out = d)
    arr <- array(0, c(S, Ttot, d))
    for (k in seq_len(d)) {
        sd0 <- sqrt(sigma_eta[k]^2 / (1 - phi[k]^2))
        arr[, 1, k] <- stats::rnorm(S, 0, sd0)
        for (t in 2:Ttot)
            arr[, t, k] <- phi[k] * arr[, t - 1, k] +
                stats::rnorm(S, 0, sigma_eta[k])
    }
    arr
}
.gen_seasonal <- function(S, Ttot, d, period) {
    stopifnot(d >= 2)
    tt <- seq_len(Ttot)
    s1 <- sin(2 * pi * tt / period); c1 <- cos(2 * pi * tt / period)
    arr <- array(0, c(S, Ttot, d))
    arr[, , 1] <- matrix(rep(s1, each = S), S, Ttot)
    arr[, , 2] <- matrix(rep(c1, each = S), S, Ttot)
    if (d > 2) arr[, , 3:d] <- stats::rnorm(S * Ttot * (d - 2))
    arr
}

.auto_period <- function(Ttot) max(12, round(Ttot / 3))
.slice_X_time <- function(X, t) {
    S <- dim(X)[1]; d <- dim(X)[3]
    Xt <- X[, t, , drop = FALSE]; dim(Xt) <- c(S, d); Xt
}
.slice_X_cov_keep <- function(X, keep, j) {
    S <- dim(X)[1]; K <- length(keep)
    Xjk <- X[, keep, j, drop = FALSE]
    dim(Xjk) <- c(S, K)
    Xjk
}

# Internal configuration for the four canonical simulation scenarios.
.scenario_config <- function(scen) {
    stopifnot(scen %in% c(1L, 2L, 3L, 4L))
    switch(as.character(scen),
           "1" = list(
               name = "Baseline",
               theta = 0.4, a = 0.05, b = 0.50,
               innov = list(dist = "norm", nu = NA),
               alpha = list(kind = "normal", mean = 0, sd = 0.5),
               omega = list(kind = "const",  val  = 0.10),
               cov_kind = list(kind = "iid"),
               burn = 200, use_bin = FALSE, bin_def = NULL,
               beta_active  = c(X1 = 0.40),
               gamma_active = c(X1 = 0.20)
           ),
           "2" = list(
               name = "Mean-driven",
               theta = 0.5, a = 0.06, b = 0.50,
               innov = list(dist = "norm", nu = NA),
               alpha = list(kind = "normal", mean = 0, sd = 0.5),
               omega = list(kind = "const",  val  = 0.10),
               cov_kind = list(kind = "seasonal"),
               burn = 220, use_bin = FALSE, bin_def = NULL,
               beta_active  = c(X1 = 0.80, X2 = 0.60),
               gamma_active = c(X1 = 0.10)
           ),
           "3" = list(
               name = "Regime Contrast (Xbin via subject median)",
               theta = 0.40, a = 0.06, b = 0.20,
               innov = list(dist = "norm", nu = NA),
               alpha = list(kind = "normal", mean = 0, sd = 0.5),
               omega = list(kind = "const",  val  = 0.01),
               cov_kind = list(kind = "ar1"),
               burn = 180, use_bin = TRUE, bin_def = "median",
               beta_active  = c(X1 = 0.20, X2 = 0.10),
               gamma_active = c(Xbin = 0.40)
           ),
           "4" = list(
               name = "Seasonal + Piecewise Regime (Xbin via L)",
               theta = 0.40, a = 0.06, b = 0.20,
               innov = list(dist = "stdt", nu = 8),
               alpha = list(kind = "normal", mean = 0, sd = 0.5),
               omega = list(kind = "const",  val  = 0.01),
               cov_kind = list(kind = "seasonal"),
               burn = 180, use_bin = TRUE, bin_def = "piecewise",
               bin_params = list(min_run_frac = 0.5, jitter_frac = 0.3),
               beta_active  = c(X1 = 0.80, X2 = 0.60),
               gamma_active = c(Xbin = 0.50)
           )
    )
}

.draw_alpha <- function(S, alpha) {
    if (alpha$kind == "normal")
        stats::rnorm(S, alpha$mean, alpha$sd)
    else stop("alpha$kind must be 'normal'")
}
.draw_omega <- function(S, omega) {
    if (omega$kind == "const") rep(omega$val, S)
    else stop("omega$kind must be 'const'")
}

#' Simulate a panel ARX-GARCHX dataset under one of four scenarios
#'
#' @param scen Scenario id in 1:4.
#' @param S Number of subjects.
#' @param T Number of post-burn-in time points (per subject).
#' @param L Mean run length for the piecewise regime (Scenario 4).
#' @param d_noise Number of nuisance covariates with zero true effect.
#' @param noise_kind Either \code{"iid"} or \code{"ar1"}.
#' @param threshold_c Threshold \eqn{c} at which the oracle exceedance
#'   probability \code{pi_true} is computed and stored.
#' @param seed Random seed.
#' @param floor_sigma2 Lower clamp on the simulated variance.
#' @return A long-format data frame with columns \code{scenario},
#'   \code{s}, \code{t}, \code{y}, oracle \code{mu} and \code{sigma},
#'   the covariate columns \code{X1..X3}, optional \code{Noise1..Noiseq},
#'   optional \code{Xbin}, and the oracle exceedance \code{pi_true}.
#'   Generating parameters are stored in \code{attr(., "params")}.
#'
#' @examples
#' set.seed(7)
#' sim <- simulate_scenario(scen = 2, S = 5, T = 80,
#'                          d_noise = 5, noise_kind = "iid", seed = 7)
#' head(sim)
#' @export
simulate_scenario <- function(
    scen        = 1L,
    S           = 8L,
    T           = 240L,
    L           = 20L,
    d_noise     = 20L,
    noise_kind  = c("iid", "ar1"),
    threshold_c = 1.0,
    seed        = NULL,
    floor_sigma2 = 1e-10
) {
    noise_kind <- match.arg(noise_kind)
    if (!is.null(seed)) set.seed(seed)
    cfg  <- .scenario_config(scen)
    Ttot <- T + cfg$burn

    X_base <- switch(cfg$cov_kind$kind,
        "iid"      = .gen_iid(S, Ttot, 3),
        "ar1"      = .gen_ar1_cov(S, Ttot, 3, phi = 0.6, sigma_eta = 1),
        "seasonal" = .gen_seasonal(S, Ttot, 3,
                                   period = .auto_period(Ttot)),
        stop("Unknown cov_kind")
    )

    X_noise <- if (d_noise > 0)
        if (noise_kind == "iid") .gen_iid(S, Ttot, d_noise)
        else .gen_ar1_cov(S, Ttot, d_noise, phi = 0.5, sigma_eta = 1)
    else array(0, c(S, Ttot, 0))

    feat_names <- c(paste0("X", 1:3),
                    if (d_noise > 0) paste0("Noise", seq_len(d_noise)))
    if (isTRUE(cfg$use_bin)) feat_names <- c(feat_names, "Xbin")
    d_total <- length(feat_names)

    X <- array(0, c(S, Ttot, d_total))
    X[, , 1:3] <- X_base
    if (d_noise > 0) X[, , 4:(3 + d_noise)] <- X_noise

    if (isTRUE(cfg$use_bin)) {
        if ((cfg$bin_def %||% "median") == "median") {
            keep_idx <- (cfg$burn + 1):Ttot
            X1_keep  <- X_base[, keep_idx, 1, drop = FALSE]
            med_s    <- apply(X1_keep, 1, stats::median, na.rm = TRUE)
            med_mat  <- matrix(med_s, nrow = S, ncol = Ttot, byrow = FALSE)
            Xbin     <- 1L * (X_base[, , 1] > med_mat)
            X[, , d_total] <- Xbin
        } else if (cfg$bin_def == "piecewise") {
            min_run <- round((cfg$bin_params$min_run_frac %||% 0.5) * L)
            jitter  <- round((cfg$bin_params$jitter_frac  %||% 0.3) * L)
            Xbin    <- .gen_regime_piecewise(Ttot, L = L,
                                             min_run = min_run,
                                             jitter  = jitter)
            X[, , d_total] <- matrix(rep(Xbin, each = S), S, Ttot)
        } else stop("Unknown bin_def: ", cfg$bin_def)
    }

    beta  <- stats::setNames(rep(0, d_total), feat_names)
    gamma <- stats::setNames(rep(0, d_total), feat_names)
    set_actives <- function(vec, act_map) {
        for (nm in names(act_map))
            if (nm %in% names(vec)) vec[nm] <- act_map[[nm]]
        vec
    }
    beta  <- set_actives(beta,  cfg$beta_active)
    gamma <- set_actives(gamma, cfg$gamma_active)

    alpha_s <- .draw_alpha(S, cfg$alpha)
    omega_s <- .draw_omega(S, cfg$omega)

    y <- mu <- sig2 <- e <- matrix(0, S, Ttot)
    for (s in seq_len(S)) {
        sig2_init <- if (cfg$a + cfg$b < 1)
            omega_s[s] / (1 - cfg$a - cfg$b)
        else omega_s[s]
        sig2[s, 1] <- sig2_init
        y[s, 1]    <- alpha_s[s]
        mu[s, 1]   <- alpha_s[s]
        e[s, 1]    <- 0
    }
    for (t in 2:Ttot) {
        Xt   <- .slice_X_time(X, t)
        mu_x <- as.numeric(Xt %*% beta)
        vx   <- as.numeric(Xt %*% gamma)
        mu[, t]   <- alpha_s + cfg$theta * y[, t - 1] + mu_x
        sig2[, t] <- omega_s + cfg$a * (e[, t - 1]^2) +
                     cfg$b * sig2[, t - 1] + vx
        sig2[, t] <- pmax(sig2[, t], floor_sigma2)
        eps_t <- if (cfg$innov$dist == "norm")
            stats::rnorm(S) else .rstdt(S, cfg$innov$nu)
        y[, t] <- mu[, t] + sqrt(sig2[, t]) * eps_t
        e[, t] <- y[, t] - mu[, t]
    }

    keep <- (cfg$burn + 1):Ttot
    df <- data.frame(
        scenario = cfg$name,
        s = rep(seq_len(S), each = length(keep)),
        t = rep(seq_along(keep), times = S),
        y = as.vector(t(y[, keep])),
        mu = as.vector(t(mu[, keep])),
        sigma = sqrt(as.vector(t(sig2[, keep])))
    )
    for (j in seq_along(feat_names))
        df[[feat_names[j]]] <- as.vector(t(.slice_X_cov_keep(X, keep, j)))

    z <- (threshold_c - df$mu) / df$sigma
    df$pi_true <- if (cfg$innov$dist == "norm")
        (1 - stats::pnorm(z))
    else
        (1 - .pstdt(z, cfg$innov$nu))

    attr(df, "params") <- list(
        cfg = cfg, beta = beta, gamma = gamma,
        threshold = threshold_c, d_noise = d_noise,
        noise_kind = noise_kind, feat_names = feat_names, L = L
    )
    df
}

test_that("simulate_scenario returns expected columns and shapes", {
    set.seed(1)
    sim <- simulate_scenario(scen = 1, S = 4, T = 60,
                             d_noise = 3, noise_kind = "iid", seed = 1)
    expect_s3_class(sim, "data.frame")
    expect_true(all(c("s","t","y","mu","sigma","pi_true",
                      "X1","X2","X3") %in% names(sim)))
    expect_equal(nrow(sim), 4 * 60)
    expect_true(all(is.finite(sim$y)))
})

test_that("lmvt returns a fitted object that print() can format", {
    set.seed(2)
    sim <- simulate_scenario(scen = 1, S = 4, T = 60,
                             d_noise = 3, noise_kind = "iid", seed = 2)
    df  <- sim[, c("s","t","y",
                   grep("^X|^Noise|^Xbin", names(sim), value = TRUE))]
    fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
                lambda_beta = 0.05, lambda_gamma = 0.05,
                maxit = 8L)
    expect_s3_class(fit, "lmvt")
    expect_named(fit, c("call","orders","penalties","ids",
                        "alpha","theta","beta","omega","a","b","gamma",
                        "xcols","standardize_X","use_x_in_variance",
                        "x_center","x_scale",
                        "converged","iters",
                        "mean_times","var_times","iter_times"),
                 ignore.order = TRUE)
    expect_length(fit$alpha, length(unique(df$s)))
    out <- capture.output(print(fit))
    expect_true(any(grepl("Penalized panel ARX-GARCHX fit", out)))
})

test_that("predict.lmvt returns finite muhat, sigmahat, and risk_g", {
    set.seed(3)
    sim <- simulate_scenario(scen = 1, S = 4, T = 60,
                             d_noise = 3, noise_kind = "iid", seed = 3)
    df  <- sim[, c("s","t","y",
                   grep("^X|^Noise|^Xbin", names(sim), value = TRUE))]
    fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
                lambda_beta = 0.05, lambda_gamma = 0.05,
                maxit = 8L)
    pr <- predict(fit, df, threshold = stats::quantile(df$y, 0.9))
    expect_true(all(c("muhat","sigmahat","risk_g","threshold_used")
                    %in% names(pr)))
    expect_true(all(is.finite(pr$muhat)))
    expect_true(all(pr$sigmahat > 0))
    expect_true(all(pr$risk_g >= 0 & pr$risk_g <= 1))
})

test_that("summary.lmvt returns model diagnostics and coefficient table", {
    set.seed(4)
    sim <- simulate_scenario(scen = 1, S = 4, T = 60,
                             d_noise = 3, noise_kind = "iid", seed = 4)
    df  <- sim[, c("s", "t", "y",
                   grep("^X|^Noise|^Xbin", names(sim), value = TRUE))]
    fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
                lambda_beta = 0.05, lambda_gamma = 0.05,
                maxit = 8L)

    sm <- summary(fit)
    expect_s3_class(sm, "summary.lmvt")
    expect_s3_class(sm$table, "data.frame")
    expect_s3_class(sm$model, "data.frame")
    expect_s3_class(sm$nonzero, "data.frame")
    expect_true(all(c("equation", "block", "parameter", "term", "estimate",
                      "abs_estimate", "selected") %in% names(sm$table)))
    expect_true(any(sm$table$parameter == "beta"))
    expect_true(any(sm$table$parameter == "gamma"))

    sm_nz <- summary(fit, include_zero = FALSE, sort_by = "abs_estimate")
    expect_true(all(sm_nz$table$selected))

    out <- capture.output(print(sm, n = 5))
    expect_true(any(grepl("Summary of varGuidTS::lmvt fit", out)))
    expect_true(any(grepl("Coefficient summary table", out)))
})

## ============================================================================
## predict.lmvt() : conditional mean, variance, and exceedance probability
##
## Reference implementation ported from `temporalVarGuid.r` in the
## variance-guided-risk-demo repository, aligned with the package's lmvt()
## API. The forward variance recursion runs subject-by-subject and
## includes a feasibility-repair step that mirrors the training-time
## guards in the (B2) gamma update, so that the variance linear predictor
## is strictly positive even on out-of-sample rows.
## ============================================================================

#' Predict from a fitted \code{lmvt} object
#'
#' Computes the conditional mean \eqn{\hat\mu_{s,t}}, conditional standard
#' deviation \eqn{\hat\sigma_{s,t}}, and the exceedance-based signal
#' metric \eqn{\hat\pi_{s,t}(c) = P(Y_{s,t} > c \,|\, F_{t-1})} on a new
#' (or held-out) panel data frame.
#'
#' @param object A fitted \code{lmvt} object.
#' @param newdata A data frame containing \code{s}, \code{t}, \code{y}
#'   columns and the same covariate columns used during fitting.
#' @param threshold Threshold \eqn{c} at which to evaluate the exceedance
#'   probability. May be (a) a single numeric, (b) a vector of length
#'   \code{nrow(newdata)}, or (c) a per-subject vector (named or aligned
#'   with the unique subjects in \code{newdata}). Defaults to the pooled
#'   90th percentile of \code{newdata$y}.
#' @param innov_g Logical; return Gaussian-innovation exceedance
#'   probability \code{risk_g}.
#' @param innov_t Logical; also return Student-t-innovation exceedance
#'   probability \code{risk_t}.
#' @param df_t Degrees of freedom for the Student-t innovation
#'   distribution.
#' @param ... Unused, kept for S3 compatibility.
#'
#' @return A data frame with columns \code{s}, \code{t},
#'   \code{yhat = muhat}, \code{muhat}, \code{sigmahat},
#'   \code{threshold_used}, and one or both of \code{risk_g}, \code{risk_t}.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' sim <- simulate_scenario(scen = 2, S = 4, T = 70, d_noise = 4,
#'                          noise_kind = "iid", seed = 1)
#' df  <- sim[, c("s", "t", "y", grep("^X|^Noise|^Xbin",
#'                                    names(sim), value = TRUE))]
#' fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
#'             lambda_beta = 0.05, lambda_gamma = 0.05, maxit = 8)
#'
#' # 1) Pooled/global threshold: one cutoff is applied to every row.
#' pr_global <- predict(fit, df, threshold = stats::quantile(df$y, 0.90))
#' head(pr_global)
#'
#' # 2) Subject-specific thresholds: use a named vector indexed by subject ID.
#' subject_cutoffs <- tapply(df$y, df$s, stats::quantile, probs = 0.75,
#'                           na.rm = TRUE)
#' pr_subject <- predict(fit, df, threshold = subject_cutoffs)
#' head(pr_subject)
#'
#' # 3) Return both Gaussian and Student-t exceedance risk scores.
#' pr_t <- predict(fit, df, threshold = subject_cutoffs,
#'                 innov_g = TRUE, innov_t = TRUE, df_t = 8)
#' head(pr_t[, c("s", "t", "muhat", "sigmahat", "risk_g", "risk_t")])
#' }
#' @export
predict.lmvt <- function(object, newdata, threshold = NULL,
                         innov_g = TRUE, innov_t = FALSE, df_t = 6, ...) {

    stopifnot(inherits(object, "lmvt"))
    stopifnot(all(c("s", "t", "y") %in% names(newdata)))
    newdata <- newdata[order(newdata$s, newdata$t), , drop = FALSE]
    ids <- object$ids

    if (is.null(threshold))
        threshold <- stats::quantile(newdata$y, 0.9, na.rm = TRUE)

    # guard: unseen subjects
    unseen <- setdiff(unique(newdata$s), ids)
    if (length(unseen))
        stop(sprintf("predict.lmvt: newdata contains unseen subject IDs: %s",
                     paste(unseen, collapse = ", ")))

    thr_per_row <- .resolve_threshold(threshold, newdata$s, ids)

    # align covariates exactly like training
    Xraw <- matrix(0, nrow = nrow(newdata), ncol = length(object$xcols))
    colnames(Xraw) <- object$xcols
    present <- intersect(object$xcols, names(newdata))
    if (length(present))
        Xraw[, match(present, object$xcols)] <-
            data.matrix(newdata[, present, drop = FALSE])
    X <- if (isTRUE(object$standardize_X))
        scale(Xraw, center = object$x_center, scale = object$x_scale)
    else
        Xraw
    attr(X, "scaled:center") <- NULL
    attr(X, "scaled:scale")  <- NULL

    Y <- as.numeric(newdata$y)

    lag_by_id <- function(v, L) {
        out <- rep(NA_real_, length(v))
        if (L <= 0L) return(v)
        for (id in ids) {
            idx <- which(newdata$s == id)
            if (length(idx) > L) {
                out[idx[(L + 1L):length(idx)]] <-
                    v[idx[1:(length(idx) - L)]]
            }
        }
        out
    }

    p <- object$orders$p; q <- object$orders$q
    r <- object$orders$r; s_ord <- object$orders$s

    # mean-side lags
    Ylags <- if (p > 0L)
        do.call(cbind, lapply(seq_len(p), function(L) lag_by_id(Y, L)))
    else NULL
    Xlags_list_mean <- lapply(0:q, function(L)
        if (L == 0L) X else apply(X, 2, lag_by_id, L = L))
    Xmean <- do.call(cbind, Xlags_list_mean)

    # variance-side design follows training choice
    Xvar <- if (isTRUE(object$use_x_in_variance))
        Xmean else matrix(0, nrow(newdata), 0)

    # conditional mean & residuals
    AR_part <- if (p > 0L)
        rowSums(Ylags %*% matrix(object$theta, ncol = 1), na.rm = TRUE) else 0
    X_part  <- as.numeric(Xmean %*% object$beta)
    muhat   <- object$alpha[match(newdata$s, ids)] + AR_part + X_part
    eps_hat <- Y - muhat
    eps_hat[!is.finite(eps_hat)] <- 0

    # forward variance recursion with feasibility repair
    u_floor <- 1e-8
    sigma2 <- rep(NA_real_, nrow(newdata))
    for (id in ids) {
        idx <- which(newdata$s == id)
        for (ii in seq_along(idx)) {
            tt <- idx[ii]

            # ARCH/GARCH parts using available lags within subject
            arch_part <- 0; garch_part <- 0
            if (r > 0L) {
                for (L in seq_len(r)) {
                    tprev <- which(newdata$t[idx] == (newdata$t[tt] - L))
                    if (length(tprev) == 1L)
                        arch_part <- arch_part +
                            object$a[L] * (eps_hat[idx[tprev]]^2)
                }
            }
            if (s_ord > 0L) {
                for (L in seq_len(s_ord)) {
                    tprev <- which(newdata$t[idx] == (newdata$t[tt] - L))
                    if (length(tprev) == 1L)
                        garch_part <- garch_part +
                            object$b[L] * sigma2[idx[tprev]]
                }
            }

            # variance linear predictor
            u0 <- object$omega[match(id, ids)] + arch_part + garch_part
            xrow_var <- if (ncol(Xvar)) Xvar[tt, ] else numeric(0)
            xg <- if (length(object$gamma))
                sum(object$gamma * xrow_var) else 0
            vlin <- u0 + xg

            # Feasibility repair: prediction-time analogue of training B2 guards
            if (!is.finite(vlin) || vlin <= u_floor) {
                if (is.finite(xg) && xg < 0) {
                    s <- (u_floor - u0) / xg  # xg < 0
                    s <- min(1, max(0, s))
                    vlin_try <- u0 + s * xg
                    if (is.finite(vlin_try) && vlin_try > u_floor) {
                        vlin <- vlin_try
                    } else {
                        vlin <- max(u0, u_floor)
                    }
                } else {
                    vlin <- max(u0, u_floor)
                }
            }

            sigma2[tt] <- vlin
        }
    }

    sigmahat <- sqrt(pmax(sigma2, u_floor))
    z <- (thr_per_row - muhat) / sigmahat

    out <- data.frame(s = newdata$s, t = newdata$t,
                      yhat = muhat, muhat = muhat,
                      sigmahat = sigmahat,
                      threshold_used = thr_per_row)
    if (innov_g) out$risk_g <- 1 - stats::pnorm(z)
    if (innov_t) out$risk_t <- 1 - stats::pt(z, df = df_t)
    out
}

# ---- threshold resolution helper --------------------------------------------

.resolve_threshold <- function(threshold, new_s, train_ids) {
    n <- length(new_s)
    # 1) single fixed threshold
    if (length(threshold) == 1L) {
        thr <- rep(as.numeric(threshold), n)
        if (!all(is.finite(thr))) stop("Non-finite threshold.")
        return(thr)
    }
    # 2) per-row vector (length == nrow(newdata))
    if (length(threshold) == n) {
        thr <- as.numeric(threshold)
        if (any(!is.finite(thr)))
            stop("Non-finite values in threshold vector.")
        return(thr)
    }
    # 3) per-subject vector (preferred)
    if (!is.null(names(threshold))) {
        thr_map <- stats::setNames(as.numeric(threshold),
                                   as.character(names(threshold)))
        thr <- thr_map[as.character(new_s)]
        if (any(is.na(thr))) {
            missing_ids <- unique(new_s[is.na(thr)])
            stop(sprintf("Thresholds missing for subjects: %s",
                         paste(missing_ids, collapse = ", ")))
        }
        if (any(!is.finite(thr)))
            stop("Non-finite values in named threshold vector.")
        return(thr)
    }
    # 4) no names: try matching by subject set size
    new_ids_unique <- unique(new_s)
    if (length(threshold) == length(new_ids_unique)) {
        thr_map <- stats::setNames(as.numeric(threshold),
                                   as.character(new_ids_unique))
        thr <- thr_map[as.character(new_s)]
        if (any(!is.finite(thr)))
            stop("Non-finite values in per-subject threshold vector.")
        return(thr)
    }
    if (length(threshold) == length(train_ids)) {
        thr_map <- stats::setNames(as.numeric(threshold),
                                   as.character(train_ids))
        thr <- thr_map[as.character(new_s)]
        if (any(is.na(thr))) {
            missing_ids <- unique(new_s[is.na(thr)])
            stop(sprintf("Thresholds (aligned to training IDs) missing for subjects: %s",
                         paste(missing_ids, collapse = ", ")))
        }
        if (any(!is.finite(thr)))
            stop("Non-finite values in per-subject threshold vector.")
        return(thr)
    }
    stop("`threshold` must be: length 1; length nrow(newdata); ",
         "or a per-subject vector whose length equals the number of ",
         "subjects (in newdata or in training). Provide names(threshold) = ",
         "subject IDs to be explicit.")
}

## ============================================================================
## summary.lmvt() : model and coefficient summary tables
## ============================================================================

# Build lagged-X term labels in the same order used by lmvt():
# cbind(X lag 0, X lag 1, ..., X lag q).
.lmvt_lagged_x_terms <- function(xcols, q, ncoef = NULL) {
    if (length(xcols) == 0L) {
        terms <- character(0)
        covars <- character(0)
        lags <- integer(0)
    } else {
        lags <- rep(0:q, each = length(xcols))
        covars <- rep(xcols, times = q + 1L)
        terms <- ifelse(lags == 0L, covars, paste0(covars, "_lag", lags))
    }

    if (!is.null(ncoef) && length(terms) != ncoef) {
        terms <- paste0("x", seq_len(ncoef))
        covars <- terms
        lags <- rep(NA_integer_, ncoef)
    }
    list(term = terms, covariate = covars, lag = lags)
}

.lmvt_coef_table <- function(object, tol = 1e-8) {
    stopifnot(inherits(object, "lmvt"))

    make_block <- function(equation, block, parameter, term, estimate,
                           subject = NA_character_, covariate = NA_character_,
                           lag = NA_integer_) {
        n <- length(estimate)
        if (n == 0L) return(NULL)
        data.frame(
            equation = rep(equation, n),
            block = rep(block, n),
            parameter = rep(parameter, n),
            term = as.character(term),
            subject = rep(subject, length.out = n),
            covariate = rep(covariate, length.out = n),
            lag = rep(lag, length.out = n),
            estimate = as.numeric(estimate),
            stringsAsFactors = FALSE
        )
    }

    ids_chr <- as.character(object$ids)
    q <- object$orders$q
    p <- object$orders$p
    r <- object$orders$r
    s_ord <- object$orders$s

    parts <- list()
    parts[[length(parts) + 1L]] <- make_block(
        equation = "mean", block = "subject_intercept", parameter = "alpha",
        term = paste0("alpha[", ids_chr, "]"), estimate = object$alpha,
        subject = ids_chr
    )

    if (length(object$theta)) {
        lag_theta <- seq_along(object$theta)
        parts[[length(parts) + 1L]] <- make_block(
            equation = "mean", block = "autoregressive", parameter = "theta",
            term = paste0("theta_lag", lag_theta), estimate = object$theta,
            lag = lag_theta
        )
    } else if (p > 0L) {
        parts[[length(parts) + 1L]] <- NULL
    }

    if (length(object$beta)) {
        xt <- .lmvt_lagged_x_terms(object$xcols, q, ncoef = length(object$beta))
        parts[[length(parts) + 1L]] <- make_block(
            equation = "mean", block = "covariate", parameter = "beta",
            term = paste0("beta[", xt$term, "]"), estimate = object$beta,
            covariate = xt$covariate, lag = xt$lag
        )
    }

    parts[[length(parts) + 1L]] <- make_block(
        equation = "variance", block = "subject_baseline", parameter = "omega",
        term = paste0("omega[", ids_chr, "]"), estimate = object$omega,
        subject = ids_chr
    )

    if (length(object$a)) {
        lag_a <- seq_along(object$a)
        parts[[length(parts) + 1L]] <- make_block(
            equation = "variance", block = "ARCH", parameter = "a",
            term = paste0("a_lag", lag_a), estimate = object$a,
            lag = lag_a
        )
    } else if (r > 0L) {
        parts[[length(parts) + 1L]] <- NULL
    }

    if (length(object$b)) {
        lag_b <- seq_along(object$b)
        parts[[length(parts) + 1L]] <- make_block(
            equation = "variance", block = "GARCH", parameter = "b",
            term = paste0("b_lag", lag_b), estimate = object$b,
            lag = lag_b
        )
    } else if (s_ord > 0L) {
        parts[[length(parts) + 1L]] <- NULL
    }

    if (length(object$gamma)) {
        xt <- .lmvt_lagged_x_terms(object$xcols, q, ncoef = length(object$gamma))
        parts[[length(parts) + 1L]] <- make_block(
            equation = "variance", block = "covariate", parameter = "gamma",
            term = paste0("gamma[", xt$term, "]"), estimate = object$gamma,
            covariate = xt$covariate, lag = xt$lag
        )
    }

    out <- do.call(rbind, parts[!vapply(parts, is.null, logical(1))])
    if (is.null(out) || !nrow(out)) {
        out <- data.frame(equation = character(0), block = character(0),
                          parameter = character(0), term = character(0),
                          subject = character(0), covariate = character(0),
                          lag = integer(0), estimate = numeric(0),
                          stringsAsFactors = FALSE)
    }
    out$abs_estimate <- abs(out$estimate)
    out$selected <- out$abs_estimate > tol
    rownames(out) <- NULL
    out
}

#' Summarize a fitted \code{lmvt} object
#'
#' Produces a compact model summary and a coefficient table for all fitted
#' parameter blocks in a penalized panel ARX--GARCHX model. The coefficient
#' table is returned as \code{summary(fit)$table} and includes subject-specific
#' intercepts/baseline variances, autoregressive and GARCH parameters, and
#' mean/variance covariate coefficients. Because \code{lmvt()} fits a penalized
#' quasi-likelihood model, the table reports estimates and variable-selection
#' indicators rather than classical standard errors or p-values.
#'
#' @param object A fitted \code{lmvt} object.
#' @param tol Absolute value used to flag a coefficient as selected/nonzero.
#' @param include_zero Logical; if \code{FALSE}, only coefficients with
#'   \code{abs(estimate) > tol} are returned in the printed/returned table.
#' @param sort_by Sorting rule for the coefficient table. Use \code{"block"}
#'   for model order, \code{"abs_estimate"} for decreasing absolute estimate,
#'   or \code{"none"} to keep construction order.
#' @param top Optional integer limiting the returned coefficient table to the
#'   first \code{top} rows after filtering and sorting.
#' @param ... Unused, kept for S3 compatibility.
#'
#' @return An object of class \code{"summary.lmvt"}, a list containing
#'   \code{model} (model-level diagnostics), \code{nonzero} (nonzero counts by
#'   parameter block), and \code{table} (the coefficient summary table).
#'
#' @examples
#' \donttest{
#' set.seed(12)
#' sim <- simulate_scenario(scen = 1, S = 4, T = 60, d_noise = 4,
#'                          noise_kind = "iid", seed = 12)
#' df  <- sim[, c("s", "t", "y", grep("^X|^Noise|^Xbin",
#'                                    names(sim), value = TRUE))]
#' fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
#'             lambda_beta = 0.05, lambda_gamma = 0.05, maxit = 8)
#' sm <- summary(fit, include_zero = FALSE)
#' sm$table
#' }
#' @export
summary.lmvt <- function(object, tol = 1e-8, include_zero = TRUE,
                         sort_by = c("block", "abs_estimate", "none"),
                         top = NULL, ...) {
    stopifnot(inherits(object, "lmvt"))
    sort_by <- match.arg(sort_by)

    full_table <- .lmvt_coef_table(object, tol = tol)
    coef_table <- full_table
    if (!isTRUE(include_zero))
        coef_table <- coef_table[coef_table$selected, , drop = FALSE]

    if (sort_by == "abs_estimate") {
        coef_table <- coef_table[order(-coef_table$abs_estimate,
                                       coef_table$equation,
                                       coef_table$parameter,
                                       coef_table$term), , drop = FALSE]
    }
    # sort_by == "block" keeps the natural model-block construction order.

    if (!is.null(top)) {
        if (length(top) != 1L)
            stop("`top` must be a positive integer when supplied.")
        top <- as.integer(top)
        if (is.na(top) || top < 1L)
            stop("`top` must be a positive integer when supplied.")
        coef_table <- coef_table[seq_len(min(top, nrow(coef_table))), , drop = FALSE]
    }
    rownames(coef_table) <- NULL

    # Counts are based on the full coefficient table, not the filtered table.
    params <- c("alpha", "theta", "beta", "omega", "a", "b", "gamma")
    nonzero <- data.frame(
        parameter = params,
        n_total = vapply(params, function(z)
            as.integer(sum(full_table$parameter == z)), integer(1)),
        n_selected = vapply(params, function(z)
            as.integer(sum(full_table$parameter == z & full_table$selected)),
            integer(1)),
        stringsAsFactors = FALSE
    )
    nonzero <- nonzero[nonzero$n_total > 0L, , drop = FALSE]
    rownames(nonzero) <- NULL

    model <- data.frame(
        metric = c("subjects", "covariates", "mean_orders_p_q",
                   "variance_orders_r_s", "lambda_beta", "lambda_gamma",
                   "x_in_variance", "iterations", "converged",
                   "mean_time_per_iter", "variance_time_per_iter",
                   "total_time_per_iter"),
        value = c(
            as.character(length(object$ids)),
            as.character(length(object$xcols)),
            sprintf("(%d, %d)", object$orders$p, object$orders$q),
            sprintf("(%d, %d)", object$orders$r, object$orders$s),
            format(object$penalties$lambda_beta, digits = 4),
            format(object$penalties$lambda_gamma, digits = 4),
            as.character(isTRUE(object$use_x_in_variance)),
            as.character(object$iters),
            as.character(isTRUE(object$converged)),
            if (length(object$mean_times))
                format(mean(object$mean_times), digits = 4) else NA_character_,
            if (length(object$var_times))
                format(mean(object$var_times), digits = 4) else NA_character_,
            if (length(object$iter_times))
                format(mean(object$iter_times), digits = 4) else NA_character_
        ),
        stringsAsFactors = FALSE
    )

    out <- list(call = object$call, model = model, nonzero = nonzero,
                table = coef_table, full_table = full_table, tol = tol,
                include_zero = include_zero, sort_by = sort_by)
    class(out) <- "summary.lmvt"
    out
}

# Internal S3 print method; registered in NAMESPACE.
print.summary.lmvt <- function(x, digits = max(3L, getOption("digits") - 3L),
                               n = 30L, ...) {
    stopifnot(inherits(x, "summary.lmvt"))
    cat("Summary of varGuidTS::lmvt fit\n")
    if (!is.null(x$call)) {
        cat("Call: ")
        print(x$call)
    }

    cat("\nModel diagnostics:\n")
    print(x$model, row.names = FALSE)

    cat("\nNonzero coefficient counts (|estimate| > ", x$tol, "):\n", sep = "")
    print(x$nonzero, row.names = FALSE)

    if (length(n) != 1L) n <- 30L
    n <- as.integer(n)
    if (is.na(n) || n < 1L) n <- 30L
    cat("\nCoefficient summary table")
    if (nrow(x$table) > n)
        cat(sprintf(" (showing first %d of %d rows)", n, nrow(x$table)))
    cat(":\n")

    tab <- x$table[seq_len(min(n, nrow(x$table))), , drop = FALSE]
    if (nrow(tab)) {
        tab$estimate <- signif(tab$estimate, digits)
        tab$abs_estimate <- signif(tab$abs_estimate, digits)
    }
    print(tab, row.names = FALSE)
    invisible(x)
}

## ============================================================================
## lmvt() : penalized panel ARX-GARCHX fit
##
## Reference implementation of the penalized panel ARX-GARCHX estimator used
## by the varGuidTS package.
##
## This implementation is a direct port of `temporalVarGuid.r` from the
## companion demo repository (variance-guided-risk-demo). It performs:
##   (A)  weighted/penalized closed-form mean-block update for
##        {alpha_s, theta, beta} via glmnet (when lambda_beta > 0) or WLS;
##   (B1) joint L-BFGS-B update of (omega_s, a, b) with gamma fixed,
##        evaluated on the strict variance mask (rows with all required
##        lags available);
##   (B2) proximal-gradient update of gamma with backtracking and
##        feasibility repair to keep the variance linear predictor
##        strictly positive;
##   (C)  forward sigma^2 recursion within each subject.
## Per-iteration timing for the mean and variance blocks is recorded.
## ============================================================================

#' Fit a penalized panel ARX-GARCHX model
#'
#' Joint maximum-quasi-likelihood fit of the conditional mean and variance
#' across a balanced panel of subjects, with subject-specific intercepts,
#' shared population coefficients, and L1 penalties on the mean and
#' variance covariate vectors.
#'
#' @param data A data frame containing columns \code{s} (subject ID),
#'   \code{t} (integer time), \code{y} (outcome), and any number of
#'   covariate columns. Other columns (e.g. \code{mu}, \code{sigma},
#'   \code{pi_true}, \code{scenario}) from \code{simulate_scenario()} are
#'   ignored.
#' @param p Autoregressive order in the mean (lags of \eqn{y}).
#' @param q Distributed-lag order for covariates \eqn{x} (used in the
#'   mean equation, and in the variance equation when
#'   \code{use_x_in_variance = TRUE}).
#' @param r ARCH order in the variance (lags of \eqn{e^2}).
#' @param s_ord GARCH order in the variance (lags of \eqn{\sigma^2}).
#' @param lambda_beta L1 penalty for the mean covariate coefficients.
#' @param lambda_gamma L1 penalty for the variance covariate coefficients.
#' @param maxit Maximum number of outer block-coordinate iterations.
#' @param tol Absolute tolerance on the QML objective for convergence.
#' @param standardize_X Logical; if \code{TRUE}, covariates are centered
#'   and scaled (per column) before fitting; means and SDs are stored on
#'   the returned object so \code{predict()} can apply the same
#'   transformation to new data.
#' @param use_x_in_variance Logical; if \code{TRUE}, the same lag block of
#'   \eqn{x} is used in both the mean and the variance equations.
#' @param phi_cap Upper bound on the GARCH stationarity sum
#'   \eqn{\sum a + \sum b}.
#' @param omega_min Lower bound on subject-specific baseline variance.
#' @param omega_cap_mult Multiplier on \code{median(eps^2)} used as the
#'   upper bound for the subject-specific baseline variance \eqn{\omega_s}
#'   in the L-BFGS-B update.
#' @param gamma_steps Number of inner proximal-gradient steps in the
#'   variance covariate (B2) update per outer iteration.
#' @param gamma_step Step size for the proximal-gradient update of
#'   \eqn{\gamma}.
#' @param verbose Logical; print per-iteration diagnostic output.
#'
#' @return An object of class \code{"lmvt"}: a list with elements
#'   \code{alpha} (subject intercepts), \code{theta} (AR coefficients),
#'   \code{beta} (mean covariate coefficients on the lagged-X block),
#'   \code{omega}, \code{a}, \code{b} (GARCH parameters), \code{gamma}
#'   (variance covariate coefficients), \code{ids}, \code{xcols},
#'   \code{x_center}, \code{x_scale}, \code{orders}, \code{penalties},
#'   \code{call}, plus diagnostics \code{converged}, \code{iters},
#'   \code{mean_times}, \code{var_times}, \code{iter_times}.
#'
#' @references Tibshirani R. (1996). Regression Shrinkage and Selection Via
#'   the Lasso. Journal of the Royal Statistical Society: Series B.
#'   doi:10.1111/j.2517-6161.1996.tb02080.x
#'
#'   Bollerslev T. (1986). Generalized Autoregressive Conditional
#'   Heteroskedasticity. Journal of Econometrics.
#'   doi:10.1016/0304-4076(86)90063-1
#'
#'   Medeiros M. C. and Mendes E. F. (2016). L1-Regularization of
#'   High-Dimensional Time-Series Models with Non-Gaussian and
#'   Heteroskedastic Errors. Journal of Econometrics.
#'   doi:10.1016/j.jeconom.2015.10.011
#'
#' @examples
#' set.seed(1)
#' sim <- simulate_scenario(scen = 1, S = 5, T = 80, d_noise = 5,
#'                          noise_kind = "iid", seed = 1)
#' df  <- sim[, c("s", "t", "y", grep("^X|^Noise|^Xbin",
#'                                    names(sim), value = TRUE))]
#' fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
#'             lambda_beta = 0.05, lambda_gamma = 0.05)
#' print(fit)
#' @export
lmvt <- function(data,
                 p = 1, q = 0,
                 r = 1, s_ord = 1,
                 lambda_beta  = 0,
                 lambda_gamma = 0,
                 maxit = 200, tol = 1e-4,
                 standardize_X     = TRUE,
                 use_x_in_variance = TRUE,
                 phi_cap     = 0.995,
                 omega_min   = 1e-6,
                 omega_cap_mult = 10,
                 gamma_steps = 8,
                 gamma_step  = 1e-3,
                 verbose     = FALSE) {

    cl <- match.call()
    stopifnot(all(c("s", "t", "y") %in% names(data)))
    data <- data[order(data$s, data$t), , drop = FALSE]
    ids  <- sort(unique(data$s))
    S    <- length(ids)
    n    <- nrow(data)

    # autodetect covariates: drop simulator-side oracle columns if present
    keep_cols <- c("s", "t", "y", "mu", "sigma", "pi_true", "scenario")
    xcols <- setdiff(names(data), keep_cols)
    if (length(xcols) == 0L)
        stop("No covariates: provide columns beyond s, t, y.")
    Xraw <- data.matrix(data[, xcols, drop = FALSE])

    if (standardize_X) {
        x_center <- colMeans(Xraw, na.rm = TRUE)
        x_scale  <- apply(Xraw, 2, stats::sd, na.rm = TRUE)
        x_scale[!is.finite(x_scale) | x_scale == 0] <- 1
        X <- scale(Xraw, center = x_center, scale = x_scale)
        attr(X, "scaled:center") <- NULL
        attr(X, "scaled:scale")  <- NULL
    } else {
        x_center <- rep(0, ncol(Xraw))
        x_scale  <- rep(1, ncol(Xraw))
        X <- Xraw
    }
    P <- ncol(X)
    Y <- as.numeric(data$y)

    # subject-aware lag of a numeric vector v by L positions within each id
    lag_by_id <- function(v, L) {
        out <- rep(NA_real_, length(v))
        if (L <= 0L) return(v)
        for (id in ids) {
            idx <- which(data$s == id)
            if (length(idx) > L) {
                out[idx[(L + 1L):length(idx)]] <-
                    v[idx[1:(length(idx) - L)]]
            }
        }
        out
    }

    # mean-side design: AR lags of y plus distributed lag block of X (lag 0..q)
    Ylags <- if (p > 0L)
        do.call(cbind, lapply(seq_len(p), function(L) lag_by_id(Y, L)))
    else NULL
    Xlags_list_mean <- lapply(0:q, function(L)
        if (L == 0L) X else apply(X, 2, lag_by_id, L = L))
    Xmean <- do.call(cbind, Xlags_list_mean)

    # variance-side design follows training choice
    if (isTRUE(use_x_in_variance)) {
        Xlags_list_var <- Xlags_list_mean
        Xvar  <- Xmean
        gamma <- rep(0, (q + 1L) * P)
    } else {
        Xlags_list_var <- list(matrix(0, n, 0))
        Xvar  <- matrix(0, n, 0)
        gamma <- numeric(0)
        lambda_gamma <- 0
    }

    # mean-block validity mask (rows with all required mean lags present)
    mask_mean <- rep(TRUE, n)
    if (p > 0L) mask_mean <- mask_mean & (rowSums(is.na(Ylags)) == 0)
    for (LL in 0:q)
        mask_mean <- mask_mean & (rowSums(is.na(Xlags_list_mean[[LL + 1L]])) == 0)

    # initial parameter values
    vl <- stats::var(Y, na.rm = TRUE)
    vl <- if (is.finite(vl) && vl > 0) vl else 1
    alpha <- tapply(Y, data$s, function(z) mean(z, na.rm = TRUE))
    alpha[is.na(alpha)] <- 0
    alpha <- as.numeric(alpha); names(alpha) <- ids
    omega <- rep(max(omega_min, vl / 10), S); names(omega) <- ids
    theta <- if (p > 0L) rep(0, p) else numeric(0)
    beta  <- rep(0, (q + 1L) * P)
    a_vec <- if (r > 0L)     rep(0.05, r)                                  else numeric(0)
    b_vec <- if (s_ord > 0L) rep((phi_cap - 0.1) / max(1L, s_ord), s_ord) else numeric(0)

    # initial residuals and sigma^2
    mu     <- alpha[match(data$s, ids)] + as.numeric(Xmean %*% beta)
    eps    <- Y - mu
    eps[!is.finite(eps)] <- 0
    sigma2 <- rep(max(vl, 1e-2), n)

    make_var_lags <- function(eps_v, sigma2_v) {
        E2lags <- if (r > 0L)
            do.call(cbind, lapply(seq_len(r), function(L) lag_by_id(eps_v^2, L)))
        else NULL
        S2lags <- if (s_ord > 0L)
            do.call(cbind, lapply(seq_len(s_ord), function(L) lag_by_id(sigma2_v, L)))
        else NULL
        list(E2lags = E2lags, S2lags = S2lags)
    }

    proj_ab <- function(a, b, cap = phi_cap) {
        a <- pmax(a, 0); b <- pmax(b, 0)
        s <- sum(a) + sum(b)
        if (s > cap && s > 0) {
            sc <- cap / s
            a <- a * sc; b <- b * sc
        }
        list(a = a, b = b)
    }

    # safe objective for (omega, a, b) with gamma fixed; evaluated only on valid_idx
    var_obj_abw <- function(par, eps, ids, s_idx, E2lags, S2lags, Xvar,
                            gamma, phi_cap, omega_min, omega_max, valid_idx) {
        BIG  <- 1e50
        Sloc <- length(unique(ids))
        n_a  <- if (is.null(E2lags)) 0L else ncol(E2lags)
        n_b  <- if (is.null(S2lags)) 0L else ncol(S2lags)

        omega <- par[1:Sloc]
        a <- if (n_a > 0L) par[(Sloc + 1L):(Sloc + n_a)] else numeric(0)
        b <- if (n_b > 0L) par[(Sloc + n_a + 1L):(Sloc + n_a + n_b)] else numeric(0)

        if (any(!is.finite(omega))) return(BIG)
        omega <- pmin(pmax(omega, omega_min), omega_max)
        a[!is.finite(a)] <- 0; b[!is.finite(b)] <- 0
        if (any(a < 0) || any(b < 0)) return(BIG)
        ssum <- sum(a) + sum(b)
        if (ssum > phi_cap && ssum > 0) { sc <- phi_cap / ssum; a <- a * sc; b <- b * sc }

        delta <- omega[s_idx]
        if (n_a > 0L) {
            if (anyNA(E2lags[valid_idx, , drop = FALSE])) return(BIG)
            delta <- delta + as.numeric(E2lags %*% a)
        }
        if (n_b > 0L) {
            if (anyNA(S2lags[valid_idx, , drop = FALSE])) return(BIG)
            delta <- delta + as.numeric(S2lags %*% b)
        }
        u <- delta + if (length(gamma)) as.numeric(Xvar %*% gamma) else 0
        v <- valid_idx
        if (any(!is.finite(u[v]))) return(BIG)
        if (any(u[v] <= 0))        return(BIG)

        val <- sum(log(u[v]) + (eps[v]^2) / u[v])
        if (!is.finite(val)) return(BIG)
        val
    }

    # per-iteration timing
    mean_times <- numeric(maxit)
    var_times  <- numeric(maxit)
    iter_times <- numeric(maxit)

    obj_prev <- Inf
    for (k in seq_len(maxit)) {

        iter_start <- proc.time()[["elapsed"]]

        ## ===== (A) Mean block ===================================================
        mean_start <- proc.time()[["elapsed"]]

        w        <- 1 / pmax(sigma2, 1e-10)
        AR_part  <- if (p > 0L) rowSums(Ylags %*% matrix(theta, ncol = 1)) else 0
        X_part   <- as.numeric(Xmean %*% beta)
        numer    <- tapply(w * (Y - AR_part - X_part), data$s, sum, na.rm = TRUE)
        denom    <- tapply(w, data$s, sum, na.rm = TRUE)
        alpha    <- as.numeric(numer / pmax(denom, 1e-12))
        names(alpha) <- ids
        alpha[!is.finite(alpha)] <- 0

        X_A   <- if (p > 0L) cbind(Ylags, Xmean) else Xmean
        y_A   <- Y - alpha[match(data$s, ids)]
        keepA <- mask_mean & is.finite(y_A) & is.finite(w) &
            is.finite(rowSums(if (is.null(dim(X_A))) cbind(X_A) else X_A))
        X_Ak  <- X_A[keepA, , drop = FALSE]
        y_Ak  <- y_A[keepA]
        wk    <- w[keepA]
        Xw    <- X_Ak * sqrt(wk)
        yw    <- y_Ak * sqrt(wk)

        if (lambda_beta > 0) {
            if (!requireNamespace("glmnet", quietly = TRUE))
                stop("Install 'glmnet' for penalized mean block (lambda_beta > 0).")
            pen_vec <- c(rep(0, p), rep(1, ncol(Xmean)))
            gfit <- glmnet::glmnet(Xw, yw, family = "gaussian",
                                   alpha = 1, lambda = lambda_beta,
                                   penalty.factor = pen_vec,
                                   standardize = FALSE, intercept = FALSE)
            coef_all <- as.numeric(as.matrix(glmnet::coef.glmnet(gfit)))[-1]
        } else {
            XtX <- crossprod(Xw); Xty <- crossprod(Xw, yw)
            coef_all <- tryCatch(
                as.numeric(solve(XtX, Xty)),
                error = function(e)
                    as.numeric(solve(XtX + 1e-8 * diag(ncol(XtX)), Xty))
            )
        }
        if (p > 0L) {
            theta <- coef_all[1:p]
            beta  <- coef_all[(p + 1L):length(coef_all)]
        } else {
            beta <- coef_all
        }

        AR_part <- if (p > 0L) rowSums(Ylags %*% matrix(theta, ncol = 1)) else 0
        X_part  <- as.numeric(Xmean %*% beta)
        mu      <- alpha[match(data$s, ids)] + AR_part + X_part
        eps     <- Y - mu
        eps[!is.finite(eps)] <- 0

        mean_times[k] <- proc.time()[["elapsed"]] - mean_start

        ## ===== Build strict variance mask =======================================
        var_start <- proc.time()[["elapsed"]]

        VL <- make_var_lags(eps, sigma2)
        E2lags <- VL$E2lags; S2lags <- VL$S2lags
        mask_var <- rep(TRUE, n)
        if (r > 0L)     mask_var <- mask_var & (rowSums(is.na(E2lags)) == 0)
        if (s_ord > 0L) mask_var <- mask_var & (rowSums(is.na(S2lags)) == 0)
        if (use_x_in_variance) {
            for (LL in 0:q)
                mask_var <- mask_var & (rowSums(is.na(Xlags_list_var[[LL + 1L]])) == 0)
        }
        valid_idx <- which(mask_var & is.finite(eps))
        if (!length(valid_idx))
            stop("No valid rows for variance update; check lags/orders.")

        ## ===== (B1) Optimize (omega, a, b) with gamma fixed =====================
        s_idx <- match(data$s, ids)
        omega_cap <- max(omega_cap_mult * stats::median(eps^2, na.rm = TRUE),
                         omega_min * 10)
        par0 <- c(omega, if (r > 0L) a_vec, if (s_ord > 0L) b_vec)
        lower <- c(rep(omega_min, S),
                   if (r > 0L)     rep(0, r),
                   if (s_ord > 0L) rep(0, s_ord))
        upper <- c(rep(omega_cap, S),
                   if (r > 0L)     rep(phi_cap, r),
                   if (s_ord > 0L) rep(phi_cap, s_ord))

        opt <- stats::optim(par0, var_obj_abw, method = "L-BFGS-B",
                            lower = lower, upper = upper,
                            control = list(maxit = 200),
                            eps = eps, ids = data$s, s_idx = s_idx,
                            E2lags = E2lags, S2lags = S2lags, Xvar = Xvar,
                            gamma = gamma, phi_cap = phi_cap,
                            omega_min = omega_min, omega_max = omega_cap,
                            valid_idx = valid_idx)

        par_star <- opt$par
        omega    <- par_star[1:S]
        if (r > 0L)     a_vec <- par_star[(S + 1L):(S + r)]
        if (s_ord > 0L) {
            off   <- S + if (r > 0L) r else 0L
            b_vec <- par_star[(off + 1L):(off + s_ord)]
        }
        abp <- proj_ab(a_vec, b_vec, phi_cap)
        a_vec <- abp$a; b_vec <- abp$b

        ## ===== (B2) Update gamma with delta fixed (prox-grad on valid_idx) =====
        delta <- omega[s_idx] +
            if (r > 0L)     as.numeric(E2lags %*% a_vec) else 0 +
            if (s_ord > 0L) as.numeric(S2lags %*% b_vec) else 0

        if (use_x_in_variance && length(gamma)) {
            v <- valid_idx
            u_floor <- 1e-8

            for (gi in seq_len(max(1L, gamma_steps))) {
                u  <- delta + as.numeric(Xvar %*% gamma)
                uv <- u[v]
                step <- gamma_step

                for (bt in 0:8) {
                    if (any(!is.finite(uv)) || any(uv <= u_floor)) {
                        shrink <- 1.0
                        repeat {
                            gamma_test <- gamma * shrink
                            uv_test <- delta[v] + as.numeric(Xvar[v, ] %*% gamma_test)
                            if (all(is.finite(uv_test)) && all(uv_test > u_floor)) {
                                gamma <- gamma_test
                                uv    <- uv_test
                                break
                            }
                            shrink <- shrink * 0.5
                            if (shrink < 1e-6) {
                                gamma <- 0 * gamma
                                uv    <- delta[v]
                                break
                            }
                        }
                        step <- step * 0.5
                    }

                    g_u      <- (1 / uv - (eps[v]^2) / (uv^2))
                    grad_gam <- as.numeric(crossprod(Xvar[v, , drop = FALSE], g_u))
                    gamma_try <- gamma - step * grad_gam
                    if (lambda_gamma > 0)
                        gamma_try <- sign(gamma_try) *
                            pmax(abs(gamma_try) - step * lambda_gamma, 0)

                    u_try <- delta[v] + as.numeric(Xvar[v, ] %*% gamma_try)
                    if (any(!is.finite(u_try)) || any(u_try <= u_floor)) {
                        step <- step * 0.5; next
                    }

                    obj_now <- sum(log(uv) + (eps[v]^2) / uv)
                    obj_try <- sum(log(u_try) + (eps[v]^2) / u_try)
                    if (is.finite(obj_try) && obj_try <= obj_now) {
                        gamma <- gamma_try; break
                    } else {
                        step <- step * 0.5
                    }
                }
            }

            uv_final <- delta[v] + as.numeric(Xvar[v, ] %*% gamma)
            if (any(!is.finite(uv_final)) || any(uv_final <= u_floor)) {
                shrink <- 1.0
                repeat {
                    gamma_test <- gamma * shrink
                    uv_test <- delta[v] + as.numeric(Xvar[v, ] %*% gamma_test)
                    if (all(is.finite(uv_test)) && all(uv_test > u_floor)) {
                        gamma <- gamma_test; break
                    }
                    shrink <- shrink * 0.5
                    if (shrink < 1e-6) { gamma <- 0 * gamma; break }
                }
            }
        } else {
            gamma <- numeric(0)
        }

        ## ===== (C) Forward sigma^2 recursion ===================================
        sigma2_new <- rep(NA_real_, n)
        for (id in ids) {
            idx <- which(data$s == id)
            for (ii in seq_along(idx)) {
                tt <- idx[ii]
                arch_part <- 0; garch_part <- 0
                if (r > 0L) {
                    for (L in seq_len(r)) {
                        tprev <- which(data$t[idx] == (data$t[tt] - L))
                        if (length(tprev) == 1L)
                            arch_part <- arch_part + a_vec[L] * (eps[idx[tprev]]^2)
                    }
                }
                if (s_ord > 0L) {
                    for (L in seq_len(s_ord)) {
                        tprev <- which(data$t[idx] == (data$t[tt] - L))
                        if (length(tprev) == 1L)
                            garch_part <- garch_part + b_vec[L] * sigma2_new[idx[tprev]]
                    }
                }
                xrow_var <- if (ncol(Xvar)) Xvar[tt, ] else numeric(0)
                vlin <- omega[match(id, ids)] + arch_part + garch_part +
                    if (length(gamma)) sum(gamma * xrow_var) else 0
                sigma2_new[tt] <- max(vlin, 1e-8)
            }
        }
        sigma2 <- sigma2_new

        var_times[k] <- proc.time()[["elapsed"]] - var_start

        # monitor (valid_idx only)
        u_now <- delta[valid_idx] +
            if (length(gamma)) as.numeric(Xvar[valid_idx, ] %*% gamma) else 0
        obj_now <- sum(log(u_now) + (eps[valid_idx]^2) / u_now)

        iter_times[k] <- proc.time()[["elapsed"]] - iter_start

        if (verbose)
            cat(sprintf(
                "iter %d: obj=%s  sum(a)+sum(b)=%.4f  mean(omega)=%.4f  t_mean=%.4fs  t_var=%.4fs\n",
                k, format(obj_now, digits = 6), sum(a_vec) + sum(b_vec),
                mean(omega), mean_times[k], var_times[k]))

        if (!is.finite(obj_now)) break
        if (abs(obj_prev - obj_now) < tol) break
        obj_prev <- obj_now
    }

    iters_used <- k
    mean_times <- mean_times[seq_len(iters_used)]
    var_times  <- var_times[seq_len(iters_used)]
    iter_times <- iter_times[seq_len(iters_used)]

    structure(list(
        call          = cl,
        orders        = list(p = p, q = q, r = r, s = s_ord),
        penalties     = list(lambda_beta = lambda_beta,
                             lambda_gamma = lambda_gamma),
        ids           = ids,
        alpha         = alpha,
        theta         = theta,
        beta          = beta,
        omega         = omega,
        a             = a_vec,
        b             = b_vec,
        gamma         = gamma,
        xcols         = xcols,
        standardize_X = standardize_X,
        use_x_in_variance = use_x_in_variance,
        x_center      = x_center,
        x_scale       = x_scale,
        converged     = (iters_used < maxit),
        iters         = iters_used,
        mean_times    = mean_times,
        var_times     = var_times,
        iter_times    = iter_times
    ), class = "lmvt")
}

# ---- S3 helpers --------------------------------------------------------------

#' @export
print.lmvt <- function(x, ...) {
    cat("Penalized panel ARX-GARCHX fit (varGuidTS::lmvt)\n")
    cat(sprintf("  Subjects:           %d\n", length(x$ids)))
    cat(sprintf("  Mean lags  (p, q):  (%d, %d)\n", x$orders$p, x$orders$q))
    cat(sprintf("  Var  lags  (r, s):  (%d, %d)\n", x$orders$r, x$orders$s))
    cat(sprintf("  Penalties (lambda_beta, lambda_gamma): (%.3g, %.3g)\n",
                x$penalties$lambda_beta, x$penalties$lambda_gamma))
    cat(sprintf("  Iterations / converged: %d / %s\n",
                x$iters, isTRUE(x$converged)))
    cat(sprintf("  Nonzero beta:  %d / %d\n",
                sum(abs(x$beta) > 1e-8), length(x$beta)))
    cat(sprintf("  Nonzero gamma: %d / %d\n",
                sum(abs(x$gamma) > 1e-8), length(x$gamma)))
    if (length(x$iter_times))
        cat(sprintf("  Mean time/iter (mean / var / total): %.4f / %.4f / %.4fs\n",
                    mean(x$mean_times), mean(x$var_times),
                    mean(x$iter_times)))
    invisible(x)
}

#' @export
coef.lmvt <- function(object, ...) {
    list(alpha = object$alpha, theta = object$theta,
         beta  = object$beta,  omega = object$omega,
         a     = object$a,     b     = object$b,
         gamma = object$gamma)
}

\name{simulate_scenario}
\alias{simulate_scenario}
\title{Simulate a panel ARX-GARCHX dataset}
\description{
Simulates a balanced panel from one of four built-in demonstration scenarios.
Returns a long-format data frame with the response \code{y}, oracle mean
and variance, the covariate block, optional regime indicator, and the oracle
exceedance probability.
}
\usage{
simulate_scenario(scen = 1L, S = 8L, T = 240L, L = 20L,
                  d_noise = 20L, noise_kind = c("iid", "ar1"),
                  threshold_c = 1.0, seed = NULL,
                  floor_sigma2 = 1e-10)
}
\arguments{
\item{scen}{Integer 1:4 selecting one of the four scenarios.}
\item{S}{Number of subjects.}
\item{T}{Number of post-burn-in time points per subject.}
\item{L}{Mean run length for the piecewise regime in Scenario 4.}
\item{d_noise}{Number of nuisance covariates with zero true effect.}
\item{noise_kind}{Either \code{"iid"} or \code{"ar1"}.}
\item{threshold_c}{Threshold \eqn{c} at which the oracle exceedance
probability is computed.}
\item{seed}{Random seed.}
\item{floor_sigma2}{Lower clamp on simulated variance.}
}
\value{
A data frame with columns \code{s}, \code{t}, \code{y}, oracle
\code{mu} and \code{sigma}, covariate columns, optional \code{Xbin},
and the oracle exceedance probability \code{pi_true}. Generating
parameters are stored in \code{attr(., "params")}.
}
\examples{
set.seed(7)
sim <- simulate_scenario(scen = 2, S = 4, T = 50,
                         d_noise = 3, noise_kind = "iid", seed = 7)
head(sim)
}
\references{
Tibshirani R. (1996). "Regression Shrinkage and Selection Via the Lasso."
\emph{Journal of the Royal Statistical Society: Series B}, 58(1), 267--288.
\doi{10.1111/j.2517-6161.1996.tb02080.x}

Bollerslev T. (1986). "Generalized Autoregressive Conditional Heteroskedasticity."
\emph{Journal of Econometrics}, 31(3), 307--327.
\doi{10.1016/0304-4076(86)90063-1}

Medeiros M. C. and Mendes E. F. (2016). "L1-Regularization of
High-Dimensional Time-Series Models with Non-Gaussian and Heteroskedastic
Errors." \emph{Journal of Econometrics}, 191(1), 255--271.
\doi{10.1016/j.jeconom.2015.10.011}
}

\name{summary.lmvt}
\alias{summary.lmvt}
\title{Summarize a fitted \code{lmvt} object}
\description{
Produces model-level diagnostics and a coefficient summary table for a
fitted penalized panel ARX-GARCHX model. The table includes
subject-specific mean intercepts, autoregressive coefficients, mean
covariate coefficients, subject-specific variance baselines, ARCH/GARCH
coefficients, and variance covariate coefficients. Because \code{lmvt()}
fits a penalized quasi-likelihood model, the table reports estimates and
variable-selection indicators rather than classical standard errors or
p-values.
}
\usage{
\method{summary}{lmvt}(object, tol = 1e-8, include_zero = TRUE,
        sort_by = c("block", "abs_estimate", "none"), top = NULL, ...)
}
\arguments{
\item{object}{A fitted \code{lmvt} object.}
\item{tol}{Absolute value used to flag a coefficient as selected/nonzero.}
\item{include_zero}{Logical; if \code{FALSE}, return only coefficients with
\code{abs(estimate) > tol}.}
\item{sort_by}{Sorting rule for the coefficient table. Use \code{"block"}
for model order, \code{"abs_estimate"} for decreasing absolute estimate,
or \code{"none"} to keep construction order.}
\item{top}{Optional positive integer limiting the returned coefficient table
to the first \code{top} rows after filtering and sorting.}
\item{...}{Unused, kept for S3 compatibility.}
}
\value{
\code{summary.lmvt()} returns an object of class \code{"summary.lmvt"}, a
list containing:
\itemize{
  \item \code{model}: model-level diagnostics;
  \item \code{nonzero}: counts of selected/nonzero coefficients by parameter block;
  \item \code{table}: the coefficient summary table after filtering/sorting;
  \item \code{full_table}: the complete coefficient table before filtering.
}
}
\examples{
\donttest{
set.seed(12)
sim <- simulate_scenario(scen = 1, S = 4, T = 60, d_noise = 4,
                         noise_kind = "iid", seed = 12)
df  <- sim[, c("s", "t", "y", grep("^X|^Noise|^Xbin",
                                   names(sim), value = TRUE))]
fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
            lambda_beta = 0.05, lambda_gamma = 0.05, maxit = 8)

# Printed summary
summary(fit)

# Programmatic coefficient table
sm <- summary(fit, include_zero = FALSE, sort_by = "abs_estimate")
sm$table
}
}
\seealso{\code{\link{lmvt}}, \code{\link{coef.lmvt}}, \code{\link{predict.lmvt}}}

\name{lmvt}
\alias{lmvt}
\title{Fit a penalized panel ARX-GARCHX model}
\description{
Joint maximum-quasi-likelihood fit of the conditional mean and variance
across a balanced panel of subjects, with subject-specific intercepts,
shared population coefficients, and L1 penalties on the mean and
variance covariate vectors.
}
\usage{
lmvt(data,
     p = 1, q = 0,
     r = 1, s_ord = 1,
     lambda_beta = 0,
     lambda_gamma = 0,
     maxit = 200, tol = 1e-4,
     standardize_X = TRUE,
     use_x_in_variance = TRUE,
     phi_cap = 0.995,
     omega_min = 1e-6,
     omega_cap_mult = 10,
     gamma_steps = 8,
     gamma_step = 1e-3,
     verbose = FALSE)
}
\arguments{
\item{data}{A data frame containing columns \code{s} (subject ID),
\code{t} (integer time), \code{y} (outcome), and any number of
covariate columns.}
\item{p}{Autoregressive order in the mean.}
\item{q}{Distributed-lag order for covariates \eqn{x}.}
\item{r}{ARCH order in the variance.}
\item{s_ord}{GARCH order in the variance.}
\item{lambda_beta}{L1 penalty for the mean covariate coefficients.}
\item{lambda_gamma}{L1 penalty for the variance covariate coefficients.}
\item{maxit}{Maximum number of outer block-coordinate iterations.}
\item{tol}{Absolute tolerance on the QML objective for convergence.}
\item{standardize_X}{Center and scale covariates before fitting.}
\item{use_x_in_variance}{Include the lagged-X block in the variance equation.}
\item{phi_cap}{Upper bound on \eqn{\sum a + \sum b}.}
\item{omega_min}{Lower bound on subject-specific baseline variance.}
\item{omega_cap_mult}{Multiplier on \code{median(eps^2)} used as the
upper bound for \eqn{\omega_s} in the L-BFGS-B update.}
\item{gamma_steps}{Inner proximal-gradient steps for \eqn{\gamma} per outer iter.}
\item{gamma_step}{Step size for the proximal-gradient update of \eqn{\gamma}.}
\item{verbose}{Logical; print per-iteration diagnostics.}
}
\value{
An object of class \code{"lmvt"}: a list with elements
\code{alpha, theta, beta, omega, a, b, gamma, ids, xcols, x_center,
x_scale, orders, penalties, call, converged, iters, mean_times,
var_times, iter_times}.
}
\references{
Tibshirani R. (1996). "Regression Shrinkage and Selection Via the Lasso."
\emph{Journal of the Royal Statistical Society: Series B}, 58(1), 267--288.
\doi{10.1111/j.2517-6161.1996.tb02080.x}

Bollerslev T. (1986). "Generalized Autoregressive Conditional Heteroskedasticity."
\emph{Journal of Econometrics}, 31(3), 307--327.
\doi{10.1016/0304-4076(86)90063-1}

Medeiros M. C. and Mendes E. F. (2016). "L1-Regularization of
High-Dimensional Time-Series Models with Non-Gaussian and Heteroskedastic
Errors." \emph{Journal of Econometrics}, 191(1), 255--271.
\doi{10.1016/j.jeconom.2015.10.011}
}
\examples{
set.seed(1)
sim <- simulate_scenario(scen = 1, S = 5, T = 80, d_noise = 5,
                         noise_kind = "iid", seed = 1)
df  <- sim[, c("s","t","y", grep("^X|^Noise|^Xbin",
                                 names(sim), value = TRUE))]
fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
            lambda_beta = 0.05, lambda_gamma = 0.05)
print(fit)
}
\seealso{\code{\link{predict.lmvt}}, \code{\link{summary.lmvt}}, \code{\link{simulate_scenario}}}

\name{coef.lmvt}
\alias{coef.lmvt}
\alias{print.lmvt}
\title{Extract or print coefficients from a fitted lmvt model}
\description{
\code{coef()} returns the list of fitted parameter blocks
(subject-specific intercepts, AR coefficients, mean covariate
coefficients, baseline variances, ARCH/GARCH parameters, and
variance covariate coefficients).

\code{print()} produces a one-screen summary of the fit.
}
\usage{
\method{coef}{lmvt}(object, ...)
\method{print}{lmvt}(x, ...)
}
\arguments{
\item{object, x}{A fitted \code{lmvt} object.}
\item{...}{Unused.}
}
\value{
For \code{coef.lmvt()}, a named list containing fitted coefficient blocks:
\describe{
\item{\code{alpha}}{subject-specific intercepts in the conditional mean model.}
\item{\code{theta}}{shared autoregressive coefficients for lagged responses.}
\item{\code{beta}}{shared coefficients for covariates in the conditional mean model.}
\item{\code{omega}}{subject-specific baseline variance parameters.}
\item{\code{a}}{shared ARCH coefficients for lagged squared residuals.}
\item{\code{b}}{shared GARCH coefficients for lagged conditional variances.}
\item{\code{gamma}}{shared coefficients for covariates in the conditional variance model.}
}
For \code{print.lmvt()}, the fitted object \code{x} is returned invisibly after
printing a short model summary.
}
\seealso{\code{\link{lmvt}}, \code{\link{summary.lmvt}}}

\name{predict.lmvt}
\alias{predict.lmvt}
\title{Predict from a fitted \code{lmvt} object}
\description{
Computes the conditional mean \eqn{\hat\mu_{s,t}}, conditional
standard deviation \eqn{\hat\sigma_{s,t}}, and the exceedance-based
risk score
\eqn{\hat\pi_{s,t}(c) = P(Y_{s,t} > c \,|\, F_{t-1})} on a new (or
held-out) panel data frame.
}
\usage{
\method{predict}{lmvt}(object, newdata, threshold = NULL,
        innov_g = TRUE, innov_t = FALSE, df_t = 6, ...)
}
\arguments{
\item{object}{A fitted \code{lmvt} object.}
\item{newdata}{A data frame containing columns \code{s}, \code{t},
\code{y} and the same covariate columns used during fitting.}
\item{threshold}{Threshold \eqn{c}. Numeric scalar, vector of length
\code{nrow(newdata)}, or per-subject vector. Defaults to the pooled
90th percentile of \code{newdata$y}.}
\item{innov_g}{Logical; return Gaussian-innovation exceedance
probability \code{risk_g}.}
\item{innov_t}{Logical; return Student-t-innovation exceedance
probability \code{risk_t}.}
\item{df_t}{Degrees of freedom for the Student-t innovation.}
\item{...}{Unused, kept for S3 compatibility.}
}
\value{
A data frame with columns \code{s}, \code{t}, \code{yhat = muhat},
\code{muhat}, \code{sigmahat}, \code{threshold_used}, and one or both
of \code{risk_g}, \code{risk_t}.
}
\examples{
\donttest{
set.seed(1)
sim <- simulate_scenario(scen = 2, S = 4, T = 70, d_noise = 4,
                         noise_kind = "iid", seed = 1)
df  <- sim[, c("s", "t", "y", grep("^X|^Noise|^Xbin",
                                   names(sim), value = TRUE))]
fit <- lmvt(df, p = 1, q = 0, r = 1, s_ord = 1,
            lambda_beta = 0.05, lambda_gamma = 0.05, maxit = 8)

# 1) Pooled/global threshold: one cutoff is applied to every row.
pr_global <- predict(fit, df, threshold = stats::quantile(df$y, 0.90))
head(pr_global)

# 2) Subject-specific thresholds: use a named vector indexed by subject ID.
subject_cutoffs <- tapply(df$y, df$s, stats::quantile, probs = 0.75,
                          na.rm = TRUE)
pr_subject <- predict(fit, df, threshold = subject_cutoffs)
head(pr_subject)

# 3) Return both Gaussian and Student-t exceedance risk scores.
pr_t <- predict(fit, df, threshold = subject_cutoffs,
                innov_g = TRUE, innov_t = TRUE, df_t = 8)
head(pr_t[, c("s", "t", "muhat", "sigmahat", "risk_g", "risk_t")])
}
}
\seealso{\code{\link{lmvt}}, \code{\link{summary.lmvt}}}

\name{varGuidTS-package}
\alias{varGuidTS-package}
\alias{varGuidTS}
\docType{package}
\title{varGuidTS: Variance-Guided Time-Series Modeling for Temporal Risk Detection}
\description{
Fits a balanced panel ARX-GARCHX model with subject-specific
intercepts, shared population coefficients, and joint L1
penalization on the conditional mean and variance. Provides an
exceedance-based risk score
\eqn{\pi_{s,t}(c) = P(Y_{s,t} > c \,|\, F_{t-1})} for personalized
temporal risk assessment from wearable-sensor and other high-frequency
panel time-series data.
}
\author{
Maintainer: Min Lu \email{luminwin@gmail.com}\cr
Authors: Zihao Wang, Min Lu
}
\references{
Tibshirani R. (1996). "Regression Shrinkage and Selection Via the Lasso."
\emph{Journal of the Royal Statistical Society: Series B}, 58(1), 267--288.
\doi{10.1111/j.2517-6161.1996.tb02080.x}

Bollerslev T. (1986). "Generalized Autoregressive Conditional Heteroskedasticity."
\emph{Journal of Econometrics}, 31(3), 307--327.
\doi{10.1016/0304-4076(86)90063-1}

Medeiros M. C. and Mendes E. F. (2016). "L1-Regularization of
High-Dimensional Time-Series Models with Non-Gaussian and Heteroskedastic
Errors." \emph{Journal of Econometrics}, 191(1), 255--271.
\doi{10.1016/j.jeconom.2015.10.011}
}
\seealso{
\code{\link{lmvt}}, \code{\link{predict.lmvt}},
\code{\link{summary.lmvt}}, \code{\link{simulate_scenario}}
}
\keyword{package}
